// GameX
// Ship Demo - Photon Class Code
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 

#include "shipdemox-ship.hpp"
#include "shipdemox-photons.hpp"

// Photons - Initialize
// Load Photon image and initialize any other resources or variables we may need for the photons
void Photons::Initialize (void)
{
	photon_img[0].Load ("Images\\photon.bmp"); // photon image needs no alpha layer because is drawn additively
}

// Photons - AddPhoton
// Add a photon to the list of particles.
int Photons::AddPhoton (float new_x, float new_y, float new_dx, float new_dy)
{
	// If maximum reached, replace an existing photon at random, otherwise add new photon at the end
	int new_photon = (num_photons==MAX_PHOTONS) ? (GameX.GetRandomInt(0,MAX_PHOTONS-1)) : (num_photons++);
	photon_list[new_photon].x = new_x;	// Initialize new photon with given or default parameters
	photon_list[new_photon].y = new_y;
	photon_list[new_photon].dx = new_dx;
	photon_list[new_photon].dy = new_dy;
	photon_list[new_photon].age = 0;		
	photon_list[new_photon].img = 0;
	return new_photon;	// Return the number of the photon just added, so it can be kept track of if desired
}

// Photons - Update
// Move all the Photons in the particle system.
// The Photons are implemented as a very simple physics engine for 
// a particle system. Each particle is advanced according to its velocity.
// Particles out of bounds are rebounded into the view.
// Particles which are too old are deleted.
void Photons::Update (void)
{									// Process each particle (photon) for updating/moving:
	for (int n=0; n<num_photons; n++) {	
		photon_list[n].age++;					// Increment age of photon, and...
		if (photon_list[n].age > MAX_AGE) {		// If photon is too old, delete it by putting the
			memcpy (&photon_list[n], &photon_list[num_photons-1], sizeof(Photon)); // last one in its place.
			num_photons--;										// And decrement the total number of photons.
		} else {		// If photon is still "young" enough:
			photon_list[n].x += photon_list[n].dx;	// Move photon by its velocity
			photon_list[n].y += photon_list[n].dy;	// After this, rebound photon off view edges:
			if (photon_list[n].x < 0 && photon_list[n].dx < 0 || photon_list[n].x > GameX.GetWidth() && photon_list[n].dx > 0)
				photon_list[n].dx = -photon_list[n].dx;
			if (photon_list[n].y < 0 && photon_list[n].dy < 0 || photon_list[n].y > GameX.GetHeight() && photon_list[n].dy > 0)
				photon_list[n].dy = -photon_list[n].dy;
		}
	}
}

// Photons - Draw
// Draw all Photons
void Photons::Draw (void)
{										// Process each particle (photon) for drawing:
	for (int n=0; n<num_photons; n++) {
		GameX.SetDrawMode (DRAW_ADD | DRAWOP_CENTERED); // Set to draw additively and centered at position.
		GameX.SetDrawScale (1.25f - (float)photon_list[n].age*0.5f/(float)MAX_AGE); // Scale to be smaller when old
		GameX.DrawImage (&photon_img[0], (int)photon_list[n].x, (int)photon_list[n].y); // Draw Photon to screen using GameX.
	}
}
