// GameX
// Ship Demo - Ship Class Code
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 

#include "shipdemox-ship.hpp"
#include "shipdemox-photons.hpp"

// Ship - Initialize
// Initialize all ship images and ship's target image
void Ship::Initialize (int num)
{
	ship_num = num;					// Initialize ship number													
	curr_img = SHIP_IMG_FLAT;		// Set current ship image
	refire = 0;
									// Set initial ship position and angle:
	if (ship_num==0) 	{pos = Vector2DF(GameX.GetWidth()/3.0f, GameX.GetHeight()/2.0f); ang = 0;}
	else				{pos = Vector2DF(GameX.GetWidth()/1.5f, GameX.GetHeight()/2.0f); ang = 180;}

									// Load Sound(s):
	photon_snd.LoadWav ("Sounds\\torpedo.wav");		
									// Load Images:
	ships.Load ("Images\\ships.gif", "Images\\ships_alpha.gif"); // Load the sprite sheet image containing the ships, and a transparency layer from a different image file
	ships.SwapColors((ColorSwapType)ship_num); // change to ship's color

	targetImg.Load ("Images\\target.tif", true); // Load the ship's target image, with transparency from the same image file
	targetImg.SwapColors((ColorSwapType)ship_num); // change to ship's color
}

// Ship - Update
// Get user input and move the ship
void Ship::Update (void)
{
	bool pressedLeft, pressedRight, pressedUp, pressedDown, pressedFire; // each is true if pressed, false if not

	if (ship_num==0) {		// Process user input for Blue Ship (num 0)...
		pressedLeft = GameX.IsKeyDown(KEY_LEFT);
		pressedRight = GameX.IsKeyDown(KEY_RIGHT);
		pressedUp = GameX.IsKeyDown(KEY_UP);
		pressedDown = GameX.IsKeyDown(KEY_DOWN);
		pressedFire = GameX.IsKeyDown(KEY_SPACE);
	} else {				// Process user input for Red Ship (num 1)...
		pressedLeft = GameX.IsKeyDown(KEY_A);
		pressedRight = GameX.IsKeyDown(KEY_D);
		pressedUp = GameX.IsKeyDown(KEY_W);
		pressedDown = GameX.IsKeyDown(KEY_S);
		pressedFire = GameX.IsKeyDown(KEY_X);
	}

	if (pressedLeft && pressedRight) pressedLeft = false, pressedRight = false; // Cancel out any
	if (pressedUp   && pressedDown ) pressedUp   = false, pressedDown  = false; // conflicting keypresses.

	if (pressedLeft) {							// Left Key Pressed
		dang += 1; curr_img++;					// Rotate ship left
		if (curr_img > SHIP_IMG_RIGHT) curr_img = SHIP_IMG_RIGHT;
	} else if (pressedRight) {					// Right Key Pressed
		dang -= 1; curr_img--;					// Rotate ship right
		if (curr_img < SHIP_IMG_LEFT) curr_img = SHIP_IMG_LEFT;
	} else {									// Otherwise..
		if (dang>0) dang *= 0.875f;				// Gradually stop the ship from
		if (dang<0) dang *= 0.875f;				// rotating
		if (curr_img < SHIP_IMG_FLAT) curr_img++;
		if (curr_img > SHIP_IMG_FLAT) curr_img--;
	}
	if(dang > 30.0f) dang = 30.0f; // limit rotation
	if(dang < -30.0f) dang = -30.0f; // limit rotation
	if (pressedUp) {							// Thrust Key Pressed, so
		vel.x += (float) cosf(ang*DEGtoRAD)*.4f;	// Increase ship velocity
		vel.y += (float)-sinf(ang*DEGtoRAD)*.4f;		
	} else if (pressedDown) {					// Reverse-Thrust Key Pressed, so
		vel.x -= (float) cosf(ang*DEGtoRAD)*.4f;	// Decrease ship velocity
		vel.y -= (float)-sinf(ang*DEGtoRAD)*.4f;
	} else {									// Otherwise, assume they mean to stop, so gradually (sort of) bring ship to a stop
		vel *= 0.9f;
	}
	if (pressedFire) {							// Fire Key Pressed
		if (refire > 3) {						// If refire counter is ready..
			Vector2DF fire = vel + Vector2DF(cosf(ang*DEGtoRAD), -sinf(ang*DEGtoRAD)) * 20.0f; // Determine direction & velocity of the new 'fired' photon
									// Add a photon to the list of photons to be processed
			extern Photons photons; // (use the global Photons object defined in shipdemox-main.cpp)
			photons.AddPhoton (pos.x, pos.y, fire.x, fire.y);
			GameX.PlaySound (&photon_snd, PLAY_REWIND, 0.5f); // Play photon torpedo sound (at 50% volume since it's loud)
			refire = 0;							// Reset the refire readiness counter
		}
	} 	
												//*** Move the ship
	ang += dang;								// Adjust angle by change-in-angle	
	pos += vel;									// Move ship position by ship's velocity
	refire++;									// Increment the refire readiness counter

	if (pos.x < 0) vel.x = -vel.x, pos.x = 0;			// If ship went out of the view,
	if (pos.y < 0) vel.y = -vel.y, pos.y = 0;			// bring it back into view.
	if (pos.x >= GameX.GetWidth())  vel.x = -vel.x, pos.x = (float)GameX.GetWidth()-1;
	if (pos.y >= GameX.GetHeight()) vel.y = -vel.y, pos.y = (float)GameX.GetHeight()-1;
}

// Ship - CollideCheck
// Do ship collision detection with another ship
void Ship::CollideCheck (Ship* ship2)
{
	target = ship2->pos - pos;					// Calculate vector to 2nd ship

	float dist = target.Length();				// Determine distance to 2nd ship
	if(dist > 0) target /= dist;				// Determine direction to 2nd ship (is normalized vector)

	if(dist < 40.0f*1.25f)						// If distance to 2nd ship is less than a certain number of pixels..
	{					
		float speed = vel.Length();				// Calculate speed of this ship
		float speed2 = ship2->vel.Length();		// Calculate speed of other ship

		// Calculate 'rebound' vector to throw ships in opposite directions 
		// (with speed equal to the sum of speeds of the two ships)
		Vector2DF rebound = target * ((speed+speed2)*0.25f);

		vel -= rebound;			// Add rebound to this ship's velocity
		ship2->vel += rebound;	// Add rebound to the other ship's velocity
	}
	target *= 100;					// Move target 100 pixels away from ship (so that we can draw it)
	ship2->target = target*-1.0f;	// Set the other ship to target this one
}

// Ship - Draw
// Draw the ship's shadow, target, and the ship itself
void Ship::Draw (void)
{
	int left = (curr_img % 5) * (80 + 1); // pixel x-value of sprite start in source "ships" image
	int top = (curr_img / 5) * (80 + 1); // pixel y-value of sprite start in source "ships" image

	// Draw the ship's shadow (by drawing the ship sprite totally black and halfway translucent)
	GameX.SetDrawShading (ColorX(0,0,0,128)); // (0,0,0,128) corresponds to (red,green,blue,opacity), each is out of 255
	GameX.SetDrawPart (left, top, left+80, top+80); // the individual ship sprite is 80x80 pixels
	GameX.SetDrawMode (DRAWOP_CENTERED);
	GameX.DrawImage (&ships, ((int)pos.x+6), ((int)pos.y+6), ang-90, 1.25f); // the +4 offsets the shadow from the ship

	// Draw the ship itself, note we draw this AFTER the shadow so it shows up on top of the shadow
	GameX.SetDrawPart (left, top, left+80, top+80);
	GameX.SetDrawMode (DRAWOP_CENTERED);
	GameX.DrawImage (&ships, (int)pos.x, (int)pos.y, ang-90, 1.25f); // (ang-90) is rotation angle, 1.25 means slightly enlarged by scaling

	// Draw ship's "target" image
	GameX.SetDrawMode (DRAWOP_CENTERED);
	GameX.DrawImage (&targetImg, (int)(pos.x+target.x), (int)(pos.y+target.y));
}